function FilesList(oElement, bSelectMultiple, oSortTypes, sCurrentRep, oAjaxplorer){
	SelectableTableRows.call(this, oElement, bSelectMultiple);
	this._sortableTable = new SortableTable(oElement, oSortTypes);
	this._bSelection = false;
	this._bUnique = false;
	this._bFile = false;
	this._bDir = false;
	this._bEditable = false;
	this._oSortTypes = oSortTypes;
	this._ajaxplorer = oAjaxplorer;
	this._pendingFile = null;
	this._currentRep = sCurrentRep;	
	
	var oThis = this;
	oElement.ownerDocument.onkeyup = function(e){
		if (e == null) e = oElement.ownerDocument.parentWindow.event;
		oThis.keyup(e);
	};
	
	Event.observe(document, "keydown", function(e){
	//oElement.ownerDocument.onkeypress = function(e){
		if (e == null) e = oElement.ownerDocument.parentWindow.event;
		return oThis.keydown(e);
	});
	
	//window.onfocus = function(){oThis.selectFirst()};
	//window.onblur = function(){oThis.blur()};
	
	if(this._currentRep != null)
	{
		this.loadXmlList(this._currentRep);
	}
	
}

FilesList.prototype = new SelectableTableRows;

FilesList.prototype.initRows = function()
{
	// Disable text select on elements
	var allItems = this.getItems();
	for(var i=0; i<allItems.length;i++)
	{
		this.disableTextSelection(allItems[i]);
	}	
}

FilesList.prototype.reload = function(pendingFileToSelect)
{
	if(this._currentRep != null) this.loadXmlList(this._currentRep, pendingFileToSelect);
}

FilesList.prototype.loadXmlList = function(repToLoad, pendingFileToSelect)
{	
	// TODO : THIS SHOULD BE SET ONCOMPLETE!
	this._currentRep = repToLoad;	
	var connexion = new Connexion();
	connexion.addParameter('mode', 'file_list');
	connexion.addParameter('rep', repToLoad);
	var oThis = this;
	this._pendingFile = pendingFileToSelect;
	this.setOnLoad();
	connexion.onComplete = function (transport){
		oThis.parseXmlAndLoad(transport.responseXML);
	}	
	connexion.sendAsync();
}

FilesList.prototype.parseXmlAndLoad = function(oXmlDoc)
{
	if( oXmlDoc == null || oXmlDoc.documentElement == null) 
	{
		this.removeOnLoad();
		return;
	}
	var root = oXmlDoc.documentElement;
	// loop through all tree children
	var cs = root.childNodes;
	var l = cs.length;
	// FIRST PASS FOR ERRORS CHECK
	for (var i = 0; i < l; i++) 
	{
		if(cs[i].tagName == "error")
		{
			this.removeOnLoad();
			alert(cs[i].firstChild.nodeValue);
			return;
		}
	}
	var items = this.getItems();
	for(var i=0; i<items.length; i++)
	{
		this.setItemSelected(items[i], false);
	}	
	this.removeCurrentLines();
	// NOW PARSE LINES
	for (var i = 0; i < l; i++) 
	{
		if (cs[i].tagName == "tree") 
		{
			this.xmlNodeToTableRow(cs[i]);
		}
	}	
	
	this.initRows();
	this.removeOnLoad();
	this._sortableTable.sortColumn = -1;
	this._sortableTable.updateHeaderArrows();
	if(this._pendingFile)
	{
		this.selectFile(this._pendingFile);
		this.hasFocus = true;
		this._pendingFile = null;
	}	
	if(this.hasFocus){
		this._ajaxplorer.foldersTree.blur();
		this._ajaxplorer.sEngine.blur();
		this._ajaxplorer.actionBar.blur();
		this.focus();
	}
	else{
		this._ajaxplorer.sEngine.blur();
		this._ajaxplorer.actionBar.blur();
		this._ajaxplorer.foldersTree.focus();
	}
	if(modal.pageLoading) modal.updateLoadingProgress('List Loaded');
}

FilesList.prototype.xmlNodeToTableRow = function(xmlNode)
{
	var newRow = document.createElement("tr");
	var tBody = this._htmlElement.getElementsBySelector("tbody")[0];
	for(i=0;i<xmlNode.attributes.length;i++)
	{
		newRow.setAttribute(xmlNode.attributes[i].nodeName, xmlNode.attributes[i].nodeValue);
	}

	["text", "filesize", "mimetype", "modiftime"].each(function(s){
		var tableCell = document.createElement("td");
		if(s == "text")
		{
			innerSpan = document.createElement("span");
			innerSpan.setAttribute("style", "cursor:default;");			
			// Add icon
			var imgString = "<img src=\"images/"+xmlNode.getAttribute('icon')+"\" ";
			imgString =  imgString + "width=\"20\" height=\"20\" align=\"ABSMIDDLE\" border=\"0\"> " + xmlNode.getAttribute(s);
			innerSpan.innerHTML = imgString;
			tableCell.appendChild(innerSpan);
		}
		else
		{
			tableCell.innerHTML = xmlNode.getAttribute(s);
		}
		newRow.appendChild(tableCell);
	});	
	
	tBody.appendChild(newRow);
}

FilesList.prototype.removeCurrentLines = function()
{
	var rows = $(this._htmlElement).getElementsBySelector('tr');
	for(i=0; i<rows.length;i++)
	{
		if(this.isItem(rows[i])) rows[i].remove();
	}
}

FilesList.prototype.setOnLoad = function()
{
	var parentObject = Position.offsetParent($(this._htmlElement));	
	addLightboxMarkupToElement(parentObject, $(this._htmlElement).getElementsBySelector('tr')[0]);
	var img = document.createElement("<img>");
	img.src = "images/loadingImage.gif";
	$(parentObject).getElementsBySelector("#element_overlay")[0].appendChild(img);
}

FilesList.prototype.removeOnLoad = function()
{
	removeLightboxFromElement(Position.offsetParent($(this._htmlElement)));
}

//
// OVERRIDE CHANGE FUNCTION
// 
FilesList.prototype.fireChange = function()
{
	this._ajaxplorer.getActionBar().update();
	this._ajaxplorer.infoPanel.update();
}

//
// OVERRIDE DBL CLICK FUNCTION
// 
FilesList.prototype.fireDblClick = function (e) {
	
	selRaw = this.getSelectedItems();
	if(!selRaw || !selRaw.length)
	{
		return; // Prevent from double clicking header!
	}
	isFile = selRaw[0].getAttribute('is_file');
	fileName = selRaw[0].getAttribute('filename');
	if(isFile == 'oui')
	{
		this._ajaxplorer.getActionBar().fireAction("download");
	}
	else
	{
		this._ajaxplorer.clickDir(fileName, this._currentRep, null);
	}
}

FilesList.prototype.getSelectedFileNames = function() {
	selRaw = this.getSelectedItems();
	if(!selRaw.length)
	{
		alert('Please select a file!');
		return;
	}
	var tmp = new Array(selRaw.length);
	for(i=0;i<selRaw.length;i++)
	{
		tmp[i] = selRaw[i].getAttribute('filename');
	}
	return tmp;
}

FilesList.prototype.getFilesCount = function() 
{	
	return this.getItems().length;
}


FilesList.prototype.fileNameExists = function(newFileName) 
{	
	var allItems = this.getItems();
	if(!allItems.length)
	{		
		return false;
	}
	for(i=0;i<allItems.length;i++)
	{
		var crtFileName = getBaseName(allItems[i].getAttribute('filename'));
		if(crtFileName && crtFileName.toLowerCase() == getBaseName(newFileName).toLowerCase()) 
			return true;
	}
	return false;
}

FilesList.prototype.selectFile = function(fileName)
{
	if(!this.fileNameExists(fileName)) 
	{
		return;
	}
	var allItems = this.getItems();
	for(var i=0; i<allItems.length; i++)
	{
		if(getBaseName(allItems[i].getAttribute('filename')) == getBaseName(fileName))
		{
			this.setItemSelected(allItems[i], true);
		}
		else
		{
			this.setItemSelected(allItems[i], false);
		}
	}
	return;
}

FilesList.prototype.getCurrentRep = function()
{
	return this._currentRep;
}

FilesList.prototype.getUserSelection = function()
{
	return new UserSelection(this.getSelectedItems(), this._currentRep);
}

FilesList.prototype.disableTextSelection = function(target)
{
	if (typeof target.onselectstart!="undefined")
	{ //IE route
		target.onselectstart=function(){return false;}
	}
	else if (typeof target.style.MozUserSelect!="undefined")
	{ //Firefox route
		target.style.MozUserSelect="none";
	}
}

FilesList.prototype.keyup = function (event)
{
	if(!this.hasFocus) return true;
	var keyCode = event.keyCode;
	if(keyCode != 38 && keyCode != 40 && keyCode != 13)
	{
		return true;
	}
	var items = this._selectedItems;
	if(items.length == 0) // No selection
	{
		return false;
	}
	var nextItem;
	var currentItem;
	var shiftKey = event['shiftKey'];
	currentItem = items[items.length-1];
	
	// UP
	if(event.keyCode == 38)
	{
		nextItem = this.getPrevious(currentItem);
	}
	//DOWN
	else if(event.keyCode == 40)
	{
		nextItem = this.getNext(currentItem);
	}
	
	if(nextItem == null)
	{
		return false;
	}
	if(!shiftKey || !this._multiple) // Unselect everything
	{ 
		for(var i=0; i<items.length; i++)
		{
			this.setItemSelected(items[i], false);
		}		
	}
	this.setItemSelected(nextItem, !nextItem._selected);
	return false;
}

FilesList.prototype.keydown = function (event)
{
	if(event.keyCode == 9) return false;
	if(!this.hasFocus) return true;
	var keyCode = event.keyCode;
	if(keyCode != 13)
	{
		return true;
	}
	var items = this._selectedItems;
	if(items.length == 0) // No selection
	{
		return false;
	}
	currentItem = items[items.length-1];
	
	//ENTER
	if(event.keyCode == 13)
	{
		for(var i=0; i<items.length; i++)
		{
			this.setItemSelected(items[i], false);
		}
		this.setItemSelected(currentItem, true);
		this.fireDblClick(null);
		return false;
	}	
}